from django.db import models
from prices import Price

from . import forms


BaseField = models.SubfieldBase('BaseField', (models.DecimalField,), {})


class PriceField(BaseField):

    description = "A field which stores a price."

    def __init__(self, verbose_name=None, currency=None, **kwargs):
        self.currency = currency
        super(PriceField, self).__init__(verbose_name, **kwargs)

    def to_python(self, value):
        if isinstance(value, Price):
            if value.currency != self.currency:
                raise ValueError('Invalid currency: %r (expected %r)' % (
                    value.currency, self.currency))
            return value
        value = super(PriceField, self).to_python(value)
        if value is None:
            return value
        return Price(value, currency=self.currency)

    def get_prep_value(self, value):
        value = self.to_python(value)
        if value is not None:
            value = value.net
        return value

    def get_db_prep_save(self, value, connection):
        value = self.to_python(value)
        if value is not None:
            value = value.net
        return connection.ops.value_to_db_decimal(value,
                                                  self.max_digits,
                                                  self.decimal_places)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        if value is not None:
            return value.net
        return super(PriceField, self).value_to_string(value)

    def formfield(self, **kwargs):
        defaults = {'currency': self.currency,
                    'form_class': forms.PriceField}
        defaults.update(kwargs)
        return super(PriceField, self).formfield(**defaults)

    def get_default(self):
        default = super(PriceField, self).get_default()
        return self.to_python(default)

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    rules = [
        ((PriceField,), [], {
            'currency': ('currency', {})
        })
    ]
    add_introspection_rules(rules, ["^django_prices\.models"])



from django import forms
from prices import Price

from .widgets import PriceInput

__all__ = ('PriceField', 'PriceInput')


class PriceField(forms.DecimalField):

    def __init__(self, currency, widget=PriceInput, *args, **kwargs):
        self.currency = currency
        if isinstance(widget, type):
            widget = widget(currency=self.currency)
        self.widget = widget
        super(PriceField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        value = super(PriceField, self).to_python(value)
        if value is None:
            return value
        return Price(value, currency=self.currency)

    def validate(self, value):
        if value is None:
            super(PriceField, self).validate(value)
        else:
            if not isinstance(value, Price):
                raise Exception('%r is not a valid price' % (value,))
            if value.currency != self.currency:
                raise forms.ValidationError(
                    'Invalid currency: %r (expected %r)' % (
                        value.currency, self.currency))
            super(PriceField, self).validate(value.net)



from django import template

register = template.Library()


@register.inclusion_tag('prices/price.html')
def gross(price):
    return {'amount': price.gross, 'currency': price.currency}


@register.inclusion_tag('prices/price.html')
def net(price):
    return {'amount': price.net, 'currency': price.currency}


@register.inclusion_tag('prices/price.html')
def tax(price):
    return {'amount': price.tax, 'currency': price.currency}



from __future__ import unicode_literals

from django import forms
from django.utils.html import escape
from django.utils.safestring import mark_safe

from prices import Price

__all__ = ['PriceInput']


class PriceInput(forms.TextInput):

    def __init__(self, currency, *args, **kwargs):
        self.currency = currency
        super(PriceInput, self).__init__(*args, **kwargs)

    def _format_value(self, value):
        if isinstance(value, Price):
            value = value.net
        return value

    def _has_changed(self, initial, data):
        if isinstance(initial, Price):
            initial = initial.net
        return super(PriceInput, self)._has_changed(initial, data)

    def render(self, name, value, attrs=None):
        result = super(PriceInput, self).render(name, value)
        result += ' %s' % (escape(self.currency),)
        return mark_safe(result)



from babel.numbers import format_currency
from django import template

register = template.Library()


@register.simple_tag
def gross(price):
    return format_currency(price.gross, price.currency)


@register.simple_tag
def net(price):
    return format_currency(price.net, price.currency)


@register.simple_tag
def tax(price):
    return format_currency(price.tax, price.currency)



from decimal import Decimal
from unittest import TestCase

from django.db import models
from prices import Price

from .models import PriceField


class Foo(models.Model):

    price = PriceField('price', currency='BTC', default='5', max_digits=9,
                       decimal_places=2)


class PriceFieldTest(TestCase):

    def test_init(self):
        foo = Foo()
        self.assertEquals(foo.price, Price(5, currency='BTC'))

    def test_get_prep_value(self):
        field = PriceField('price', currency='BTC', default='5', max_digits=9,
                           decimal_places=2)
        self.assertEquals(field.get_prep_value(Price(5, currency='BTC')),
                          Decimal(5))

    def test_to_python(self):
        field = PriceField('price', currency='BTC', default='5', max_digits=9,
                           decimal_places=2)
        self.assertEqual(field.to_python(7), Price(7, currency='BTC'))



#! /usr/bin/env python
from setuptools import setup

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Software Development :: Libraries :: Python Modules']

setup(name='django-prices',
      author='Mirumee Software',
      author_email='hello@mirumee.com',
      description='Django fields for the prices module',
      license='BSD',
      version='0.3',
      url='http://satchless.com/',
      packages=['django_prices', 'django_prices.templatetags'],
      include_package_data=True,
      classifiers=CLASSIFIERS,
      install_requires=['django', 'prices>=0.5,<0.6a0'],
      platforms=['any'],
      test_suite='django_prices.tests',
      zip_safe=False)
